# -*- coding: utf-8 -*-

# -*- coding: utf-8 -*-
#***************************************************************************
 #*   Copyright (C) 2004 by Lionel Roubeyrie aka joshua_fr                                *
 #*   Lionel Roubeyrie                                                    *
 #*                                                                         *
 #*   This program is free software; you can redistribute it and/or modify  *
 #*   it under the terms of the GNU General Public License as published by  *
 #*   the Free Software Foundation; either version 2 of the License, or     *
 #*   (at your option) any later version.                                   *
 #*                                                                         *
 #*   This program is distributed in the hope that it will be useful,       *
 #*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 #*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 #*   GNU General Public License for more details.                          *
 #*                                                                         *
 #*   You should have received a copy of the GNU General Public License     *
 #*   along with this program; if not, write to the                         *
 #*   Free Software Foundation, Inc.,                                       *
 #*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 #***************************************************************************
 
 # The fstab Editor
 
import os
from qt import *
##~ from kfile import KFileDialog
from editor_frm import editor_frm

from qttable import *
from Jwidgets import *
from fstab_parser import FDReader

global FSCK_LIST
FSCK_LIST = ['not checked', 'checked first', 'last checked'] #Strings put in the combo

class entry_editor(editor_frm):
    def __init__(self,parent = None,name = None,modal = 0,fl = 0, entry = None, fd_dir=None):
        editor_frm.__init__(self,parent,name,modal,fl)
        self.FD_DIR = fd_dir
        self.TYPE_LIST = os.listdir(fd_dir)
        self.initCombos()
        self.entry = entry
        fs_type = entry['fs_vfstype']
        self.setElements(self.entry, fs_type)
        
    def initCombos(self):
        for str in FSCK_LIST:
            self.fsck_choice.insertItem(str)
        for str in self.TYPE_LIST:
            self.type_choice.insertItem(str)

    def ok_btn_clicked(self):
        entry = self.getElements()
        msg = ''
        if (entry['fs_spec'] == None) or (entry['fs_spec'] == ''):
            msg += 'A device must be declared. \n'
        if (entry['fs_file'] == None) or (entry['fs_file'] == ''):
            msg += 'A filesystem must be declared. \n'
        if (entry['fs_mntops'] == None) or (entry['fs_mntops'] == ''):
            msg += 'A least one option must be declared.\n'
        if (msg != ''):
            QMessageBox.warning( self, "Warning",  msg)
        else:
            self.accept()

    def cancel_btn_clicked(self):
        pass

    def type_choice_activated(self,fs_type):
        self.setElements(self.entry, fs_type)

    def device_btn_clicked(self):
        file = QFileDialog.getOpenFileName("/dev", "*")
        if file:
            self.device_field.setText(file)

    def mountpoint_btn_clicked(self):
        dlg = QFileDialog("/mnt", "*", None, "filedialog", 1 )
##~         dlg = KFileDialog(QString("/mnt"), QString("*"), self, "filedialog", 1 )
        dlg.setCaption( "Select a directory" )
        dlg.setMode(QFileDialog.DirectoryOnly)
        if dlg.exec_loop():
            self.mountpoint_field.setText(dlg.selectedFile())
    
    def add_btn_clicked(self):
        self.passOption(source_table=self.options_table_B, target_table=self.options_table)
                
    def remove_btn_clicked(self):
        self.passOption(source_table=self.options_table, target_table=self.options_table_B)
    
    def passOption(self, source_table, target_table):
        '''Passes the items (entry) selected in the source table to the target table.'''
        target   = target_table # target table
        source   = source_table #source table
        scurrow  = source.currentRow()
        if source.numRows() == 0:
            pass
        else:
            i_label  = source.item(scurrow, 0)
            i_value  = source.item(scurrow, 1)
            i_desc   = source.item(scurrow, 2)
            source.takeItem(i_label)
            source.takeItem(i_value)
            source.takeItem(i_desc)
            
            trow = target.numRows()
            target.insertRows(trow, 1)
            target.setItem(trow, 0, i_label)
            target.setItem(trow, 1, i_value)
            target.setItem(trow, 2, i_desc)
            
            source.removeRow(scurrow)
            target.sortColumn(0,1,1)
            source.sortColumn(0,1,1)
        
    def setElements(self, entry, fs_type):
        '''Sets the differents values in the good fields in the frame.'''
        self.delAllRows()
        self.extractFD(fs_type) #Initialises the options associate with the filesystem
        self.device_field.setText(entry['fs_spec'])
        self.mountpoint_field.setText(entry['fs_file'])
        self.type_choice.setCurrentText(fs_type)
        self.dump.setChecked(int(entry['fs_freq']))
        self.fsck_choice.setCurrentItem(int(entry['fs_passno']))
        self.comment_field.setText(entry['fs_comment'])
        self.setOptions(opts=entry['fs_mntops'])
    
    def getElements(self):
        '''Returns a dictionnary with the modified entry after user modifications.'''
        entry = {}
        entry['fs_spec'] = str(self.device_field.text()).strip()
        entry['fs_file'] = str(self.mountpoint_field.text()).strip()
        entry['fs_vfstype'] = str(self.type_choice.currentText()).strip()
        entry['fs_freq'] = int(self.dump.isChecked())
        entry['fs_passno'] = int(self.fsck_choice.currentItem())
        entry['fs_comment'] = str(self.comment_field.text()).strip()
        entry['fs_mntops'] = self.getOptions()
        return entry
        
    def extractFD(self, fs_type):
        '''Sets all lines found in all files descriptors in file_desc'''
        reader = FDReader(self.FD_DIR)
        self.file_desc = []
        if (fs_type != '') :
            type_list = ['base',fs_type] #There's can be only one file system type selected
            for type in type_list:
                fdfile = reader.getFDFile(str(type))
                if fdfile is not None :
                    [self.file_desc.append(r.strip()) for r in fdfile]
        else:
            self.file_desc = []
    
    def delAllRows(self):
        '''Deletes all rows in the two tables '''
        nr = self.options_table.numRows()
        for i in range(nr,-1,-1):
            self.options_table.removeRow(i)
        nrb = self.options_table_B.numRows()
        for j in range(nrb,-1,-1):
            self.options_table_B.removeRow(j)

    def setOptions(self, opts=' '):
        '''Sets the options in the options_table or in the options_table_B with the good options coming with the type of filesystem.
        A line in the options_table(_B) is like that : label(text or combobox)|value(text or combobox)|description(text).
        A line in the file descriptor of the filesystem is like that :
        fd_label_type|fd_label|fd_value_type|fd_value|description with:
        fd_label_type and fd_value_type can be : 0 for a editable QLineEdit, 1 for no-editable QLineEdit, 2 for a QComboTableItem(no editable)
        fd_label and fd_value are simple strings if type is 0 or 1, a list of strings if 2'''
        opts_dic = self.setOptsInDic(str(opts))
        #Creates rows in the options_table
        for row in range(len(self.file_desc)):
            try:
                fd_line      = self.file_desc[row].split('|')
                fd_label_type  = int(fd_line[0])
                fd_label       = fd_line[1]
                fd_value_type  = int(fd_line[2])
                fd_value       = fd_line[3]
                desc           = fd_line[4]
                #Test to see if the label's option is in the file descriptor
                is_in_fd = None
                for opt_label in opts_dic.keys():
                    if opt_label in fd_label.split(','):
                        is_in_fd = opt_label 
                if is_in_fd is not None: 
                    t   = self.options_table
                    lab = is_in_fd
                    val = opts_dic[lab]
                else:
                    t   = self.options_table_B
                    lab = fd_label
                    val = fd_value
                t.insertRows(t.numRows(), 1)
                #Sets the label(s)
                if (fd_label_type == 0) or (fd_label_type == 1):
                    i_label = JTableItem(t, JTableItem.Never, lab)
                if (fd_label_type == 2):
                    label_list = QStringList()
                    [label_list.append(str(x)) for x in fd_label.split(',')]
                    i_label = JComboTableItem(t, label_list, 0)
                    i_label.setCurrentItem(lab)
                t.setItem(t.numRows()-1, 0, i_label)
                #Sets the value(s)
                if (fd_value_type == 0):
                    i_value = JTableItem(t, JTableItem.Always, val)
                if (fd_value_type == 1):
                    i_value = JTableItem(t, JTableItem.Never, val)
                if (fd_value_type == 2):
                    value_list = QStringList()
                    [value_list.append(str(x)) for x in fd_value.split(',')]
                    i_value = JComboTableItem(t, value_list, 0)
                    i_value.setCurrentItem(val)
                t.setItem(t.numRows()-1, 1, i_value)
                #Sets the description
                t.setText(t.numRows()-1, 2, desc)
            except:
                pass
        #Some table adjustments
        for t in [self.options_table, self.options_table_B]:
            for col in range(t.numCols()):
                t.adjustColumn(col)
            t.setColumnReadOnly(2, 1)
            t.sortColumn(0,1,1)
        
    def getOptions(self):
        '''Returns a string of the options of the options_table'''
        opts_list = []
        t         = self.options_table
        ot_num    = t.numRows()
        for row in range(ot_num):
            i_label  = t.item(row, 0)
            i_value  = t.item(row, 1)
            label = i_label.currentText()
            if label[-1] == '=':
                value = i_value.currentText()
                opt = str(label).strip() + str(value).strip()
            else:
                opt = str(label).strip()
            opts_list.append(opt)
        return ','.join(opts_list)
    
    def setOptsInDic(self, opts):
        '''Returns a dictionnary of {label:value} of options for an easier manipulation'''
        opts_dic  = {}
        opts_list = str(opts).split(',')
        for opt in opts_list: #Reads each pairs of option/value in the option block
            opt = opt.split('=')
            if (len(opt) > 1): #the option is like : label=value, then we have to split it 
                label = opt[0] + '='
                value = opt[1]
            else : # the option is just a label
                label = opt[0]
                value = ''
            opts_dic[label]=value
        return opts_dic
