# -*- coding: utf-8 -*-
#***************************************************************************
 #*   Copyright (C) 2004 by Lionel Roubeyrie aka joshua_fr                                *
 #*   Lionel Roubeyrie                                                    *
 #*                                                                         *
 #*   This program is free software; you can redistribute it and/or modify  *
 #*   it under the terms of the GNU General Public License as published by  *
 #*   the Free Software Foundation; either version 2 of the License, or     *
 #*   (at your option) any later version.                                   *
 #*                                                                         *
 #*   This program is distributed in the hope that it will be useful,       *
 #*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 #*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 #*   GNU General Public License for more details.                          *
 #*                                                                         *
 #*   You should have received a copy of the GNU General Public License     *
 #*   along with this program; if not, write to the                         *
 #*   Free Software Foundation, Inc.,                                       *
 #*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 #***************************************************************************
 
 #Parser for the fstab and mtab files

import os

def getFDList(fd_dir):
    '''Returns a list of all filesytems supported by kouvert, one file descryptor 
    by filesystem'''
    return os.listdir(fd_dir)

class Record:

    def __init__(self, fstab='fstab', mtab='mtab'):
        '''Reads the fstab file.'''
        self.fstab = fstab
        self.mtab =  mtab
        self.fstab_list = self.readTab(self.fstab)
        self.mtab_list = self.readTab(self.mtab)

    def readTab(self, file):
        '''Reads the tab file (fstab or mtab) and returns a list of entries found'''
        list = []
        tab = open(file, 'r')		
        for line in tab:
##~             if (line[0] != '#') and (line != '\n'):
            list.append(line.strip())
        tab.close()
        return list
        
    def getRecordList(self):
        '''Returns a list wich contains all the records.'''
        #print ' %i records found in fstab.' %len(self.fstab_list)
        record_list = []
        for linenb in range(len(self.fstab_list)):
            if len(self.fstab_list[linenb]) > 0:
                if (self.fstab_list[linenb][0] != '#') and (self.fstab_list[linenb] != '\n'):
                    record_list.append(self.getRecord(linenb))
        return record_list
        
    def getRecord(self, linenb):
        '''Returns a formated record given at the position pos'''
        #looks at the previous line for searching a comment
        try:
            if self.fstab_list[linenb-1][0] == '#':
                comment = self.fstab_list[linenb-1][1:] 
        except:
            pass
         #if founds comment in the same line, removes it
        rec = self.fstab_list[linenb].split('#')
        rec = rec[0].split()
        record = {}
        try: record['fs_spec'] = rec[0]
        except: record['fs_spec'] = ''
        try: record['fs_file'] = rec[1]
        except: record['fs_file'] = ''
        try: record['fs_vfstype'] = rec[2]
        except: record['fs_vfstype'] = ''
        try: record['fs_mntops'] = rec[3]
        except: record['fs_mntops'] = ''
        try: record['fs_freq'] = rec[4]
        except: record['fs_freq'] = ''
        try: record['fs_passno'] = rec[5]
        except: record['fs_passno'] = ''        
        try: record['fs_comment'] = comment
        except: record['fs_comment'] = ''
        return record

    def isMounted(self, fs_spec, fs_file):
        '''Returns 1 if a device is mounted (according to mtab),
        0 if not.'''
        ismounted = 0
        #Refresh the mtab_list
        self.mtab_list = self.readTab(self.mtab)
        #is it mounted?
        for record in self.mtab_list:
            record = record.split()
            if (record[0] == fs_spec) and (record[1] == fs_file):
                ismounted = 1
        return ismounted        

    def setRecords(self, fstab, records):
        file=open(fstab, 'w')
        for rec in records:
            if rec['fs_comment'].strip() != '':
                comment =  "#"+rec['fs_comment']
                file.write(comment+'\n')
            line = [rec['fs_spec'], rec['fs_file'], rec['fs_vfstype'], rec['fs_mntops'], rec['fs_freq'], rec['fs_passno']]
            file.write('\t'.join(line)+'\n')
        file.close()    
    
class FDReader:
    def __init__(self, fd_dir):
        self.FD_DIR = fd_dir
    def getFDFile(self, file):
        '''Reads the file descriptor of the filesystem given in file and returns a list of entries found'''
        file = os.path.join(self.FD_DIR, file)
        list = []
        try:
            fs_file = open(file, 'r')
        except:
            return None
        for line in fs_file:
            if (line[0] != '#') and (line != '\n') and (len(line)>0):
                list.append(line.strip())
        fs_file.close()
        return list
