#!/usr/bin/env python
# -*- coding: utf-8 -*-
#***************************************************************************
 #*   Copyright (C) 2004 by Lionel Roubeyrie aka joshua_fr                                *
 #*   Lionel Roubeyrie                                                    *
 #*                                                                         *
 #*   This program is free software; you can redistribute it and/or modify  *
 #*   it under the terms of the GNU General Public License as published by  *
 #*   the Free Software Foundation; either version 2 of the License, or     *
 #*   (at your option) any later version.                                   *
 #*                                                                         *
 #*   This program is distributed in the hope that it will be useful,       *
 #*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 #*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 #*   GNU General Public License for more details.                          *
 #*                                                                         *
 #*   You should have received a copy of the GNU General Public License     *
 #*   along with this program; if not, write to the                         *
 #*   Free Software Foundation, Inc.,                                       *
 #*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 #***************************************************************************
#This is the base of Kouvert

import sys, os,string
import shutil
try:
    v = sys.version_info
    if (v[0]>=2) and (v[1]>=3):pass
    else:
        print('Bad Python version. You must use Python 2.3.0 or above.')
        sys.exit(1)
except:
    print('It seems that you use a old version of Python. Please upgrade for Python 2.3.0 or above.')
    sys.exit(1)

try:
    from qt import *
except:
    print('PyQt is not installed. Qt version 3.2 or above and the appropriate version of PyQt must be installed for using Kouvert.')
    sys.exit(1)

import fstab_parser
from kouvert_frm import kouvert_frm
from editor import entry_editor


global VERSION, AUTHOR, EMAIL, TEXT, default_entry
VERSION = 'Kouvert version 0.3'
AUTHOR='Lionel Roubeyrie aka joshua_fr'
EMAIL='lionel.roubeyrie@chello.fr'
TEXT='Kouvert is a fstab (filesystem table) editor with some facilities. \n It comes with absolutly no warranty, a bad usage (or an internal bug) \n \
may cause your system unstable or unusable.'
default_entry = {
'fs_spec' : 'none',
'fs_file' : '/tmp/kouvert',
'fs_vfstype' : 'auto',
'fs_mntops' : 'defaults',
'fs_freq' : '0',
'fs_passno' : '0',
'fs_comment' :'Comment me !'}
        
class main_kouvert(kouvert_frm):

    def __init__(self,parent = None,name = None,fl = 0):
        self.FSTAB = '/etc/fstab'
        self.MTAB = '/etc/mtab'
        self.FILESYS = '/etc/filesystems'
        self.FD_DIR = 'fd_options'
        
        #Read the files
        try:
            self.parser = fstab_parser.Record(self.FSTAB,self.MTAB)
        except Exception, detail:
            print("An error occured when opening fstab and mtab files : %s" % str(detail))
            sys.exit(1)
        
        #Create the main window
        kouvert_frm.__init__(self,parent,name,fl)
        #Appends the whatsthis button mode at the toolbar and the menubar
        QWhatsThis.whatsThisButton(self.toolBar)
        self.helpMenu.insertItem( "What's &This", self, SLOT("whatsThis()"), Qt.SHIFT+Qt.Key_F1)
        #Get the vertical header 
        self.rowlabel=self.fs_table.verticalHeader()
        #Initialize the file used for fstab
        self.file = self.FSTAB
        #Clear the table
        self.clearFs_table()
        #Puts the entries in the fs_table
        self.putEntries(self.FSTAB, self.MTAB)
        #Are they mounted?
        self.checkMounted()
        #The "is mounted" column must be read only
        self.fs_table.setColumnReadOnly(7, 1)
        #We can move the rows for reordering the fstab
        self.fs_table.setRowMovingEnabled(True)
        #if a backup file not exist, copy /etc/fstab to /etc/fstab.bck, else the user can make it
        if os.path.exists('/etc/fstab.bck'):
            q = QMessageBox.question(self, "Backup", "A backup file already exists. Ovewrite it with the present fstab file?", \
            QMessageBox.Yes, QMessageBox.No)
            if q == QMessageBox.Yes:
                self.backup()  
        else:
            b = self.backup()
            if b:
                msg = 'I have made a backup copy of your /etc/fstab file in /etc/fstab.bck. Remember it if you go in trouble!'
                QMessageBox.information( self, "Backup",  msg)

    def backup(self):
        b = 1
        try:
            shutil.copy(self.FSTAB, '/etc/fstab.bck')
        except Exception, detail:
            QMessageBox.warning( self, "Kouvert :: Error in backup",  str(detail))
            b = 0
        return b #the state of the backup (1=Ok, 0= Not Ok)
            
    def fileNew(self):
        self.clearFs_table()
        self.fs_table.insertRows(0, 1)
        self.setEntry(0, default_entry)
        self.file = None

    def fileOpen(self):
        dlg = QFileDialog("/", "*", None, "filedialog", 1 )
        dlg.setCaption( "Select a file" )
        dlg.setMode(QFileDialog.ExistingFile)
        if dlg.exec_loop():
            file_name = str(dlg.selectedFile())
            self.clearFs_table()
            self.putEntries(fstab=file_name, mtab=self.MTAB)
            self.file = file_name

    def fileSave(self):
        if self.file is not None: #The file is already opened
            self.savein(self.file)
        else:
            dlg = QFileDialog("/", "*", None, "filedialog", 1 )
            dlg.setCaption( "Save file" )
            dlg.setMode(QFileDialog.AnyFile)
            if dlg.exec_loop():
                file_name = str(dlg.selectedFile())
                s = self.savein(file_name)
                if s: #The file is written
                    self.file = file_name
                

    def fileSaveAs(self):
        dlg = QFileDialog("/", "*", None, "filedialog", 1 )
        dlg.setCaption( "Save file" )
        dlg.setMode(QFileDialog.AnyFile)
        if dlg.exec_loop():
            file_name = str(dlg.selectedFile())
            self.savein(file_name)

    def helpAbout(self):
        msg = VERSION +'\n'
        msg += AUTHOR +'\n'
        msg += TEXT
        QMessageBox.about( self, "About kouvert",  msg)

    def delete_btn_clicked(self):
        '''Deletes the current row.'''
        self.fs_table.removeRow(self.fs_table.currentRow())

    def mount_btn_clicked(self):
        self.mount(action='mount')
        
    def umount_btn_clicked(self):
        self.mount(action='umount')
        
    def mount(self, action):
        '''mount or unmount a device'''
        cr = self.fs_table.currentRow()
        if cr == -1:
            return
        entry = self.getEntry(cr)
##~         ismounted = str(self.fs_table.text(cr, 7))
        if (action == 'umount'):
            cmd = 'umount %s' % str(entry['fs_file'])
        if (action == 'mount'):
            cmd = 'mount -t %s -o %s %s %s' % (str(entry['fs_vfstype']), str(entry['fs_mntops']), str(entry['fs_spec']), str(entry['fs_file']))
        #We call popen3 and we read outputs
        (child_stdin, child_stdout, child_stderr)=os.popen3(cmd)
        stdout = child_stdout.read()
        stderr = child_stderr.read()
        if (stdout != '') or (stderr != ''):
            msg = ''
            if (stdout != ''): msg += ' For your information : %s \n' % (stdout)
            if (stderr != ''): msg += ' Problem : %s \n' % (stderr)
            QMessageBox.warning( self, "umount infos",  msg)
        self.checkMounted()
        
    def editor_btn_clicked(self):
        '''Calls the entry_editor for modifying a selected entry.'''
        cr = self.fs_table.currentRow()
        entry = self.getEntry(cr)
        self.openEditor(cr, entry)
    
    def add_btn_clicked(self):
        '''Adds a new entry by calling the editor and passing to it some
        defaults elements.'''
        self.openEditor(-1, default_entry)

    def openEditor(self, row, entry):
        '''Open the editor for the entry declared at the row'''
        fdlist = os.listdir(self.FD_DIR)
        if entry['fs_vfstype'] not in fdlist:
            msg = 'I cannot serve this entry to you, because the filesystem \n is not currently supported by Kouvert. \n Please modify it directly.'
            QMessageBox.information( self, "Not supported",  msg)
        else:
            try:
                entry_dlg = entry_editor(parent = self,name = None,modal = 1,fl = 0, entry=entry, fd_dir=self.FD_DIR)
                entry_dlg.show()
                if entry_dlg.exec_loop() == QDialog.Accepted:
                    new_entry = entry_dlg.getElements()
                    if (row ==-1): #It's a new entry, then we append an new row at the table
                        row = self.fs_table.numRows()
                        self.fs_table.insertRows(row, 1)
                    self.setEntry(row, new_entry)
            except Exception, detail:
                msg = 'Invalid entry founds. Check and retry.\n Error : %s' %str(detail)
                print msg
                QMessageBox.warning( self, "Warning",  msg)
    
    def getEntry(self, rs):
        t = self.fs_table
        try:
            rec = {}
            rec['fs_spec'] = str(t.text(rs, 0))
            rec['fs_file'] = str(t.text(rs, 1))
            rec['fs_vfstype'] = str(t.text(rs, 2))
            rec['fs_mntops'] = str(t.text(rs, 3))
            rec['fs_freq'] = str(t.text(rs, 4))
            rec['fs_passno'] = str(t.text(rs, 5))
            rec['fs_comment'] = str(t.text(rs, 6))
        except:
            rec = None
        return rec
    
    def setEntry(self, row, rec):
        t = self.fs_table
        #Sets texts
        t.setText(row, 0, str(rec['fs_spec']))
        t.setText(row, 1, str(rec['fs_file']))
        t.setText(row, 2, str(rec['fs_vfstype']))
        t.setText(row, 3, str(rec['fs_mntops']))
        t.setText(row, 4, str(rec['fs_freq']))
        t.setText(row, 5, str(rec['fs_passno']))
        t.setText(row, 6, str(rec['fs_comment']))
        self.checkMounted()
        dev = self.getDevice(rec)
        if dev is not None:
            dev = 'icones/'+dev+'.png'
            self.rowlabel.setLabel(row, QIconSet(QPixmap(dev)),'')
            
    def putEntries(self, fstab, mtab):
        '''Set all entries (records) found in the fstab'''
        #Read the files
        try:
            self.parser = fstab_parser.Record(fstab, mtab)
        except Exception, detail:
            QMessageBox.warning( self, "An error occured when opening fstab and mtab files",  detail)
            print("An error occured when opening fstab and mtab files : %s" % str(detail))
            sys.exit(1)
        record_list = self.parser.getRecordList()
        #Put the entries in the fs_table
        for record in record_list:
            nr = self.fs_table.numRows()
            self.fs_table.insertRows(nr, 1)
            self.setEntry(nr, record)
        return 1
                
    def checkMounted(self):
        '''Checks if the devices are mounted and write state'''
        entnum = self.fs_table.numRows()
        for entry in range(entnum):
            fs_spec = self.fs_table.text(entry, 0)
            fs_file = self.fs_table.text(entry, 1)
            if self.parser.isMounted(fs_spec, fs_file):
                self.fs_table.setText(entry, 7, 'yes')
            else:
                self.fs_table.setText(entry, 7, 'no')
    
    def clearFs_table(self):
        nr = self.fs_table.numRows()
        for i in range(nr):
            self.fs_table.removeRow(nr-1)
            nr = self.fs_table.numRows()
    
    def savein(self, file_name):
        '''Saves all entries found in the fs_table in the file file.'''
        entries = []
        [entries.append(self.getEntry(entry)) for entry in range(self.fs_table.numRows())]
        if len(entries)>0:
            try:
                self.parser.setRecords(file_name, entries)
            except Exception, detail:
                QMessageBox.warning( self, "Warning",  str(detail))
                return None
        else: QMessageBox.warning( self, "Warning",  "At least one entry please !")
        return 1
    
    def getDevice(self, rec):
        device = 'unknown'
        if (str(rec['fs_vfstype']) == 'nfs') or (string.find(str(rec['fs_vfstype']),':') != -1):
            device='nfs'
        else:
            try:
                s= os.stat(str(rec['fs_spec']))
                major = os.major(s.st_rdev)
            except Exception,detail:       
                return device
            if major == 2:
                device='floppy'
            if major == 3:
                try:
                    i = int(str(rec['fs_spec'])[-1])#if no error occur, then it's an integer, then it's a disk partition
                    device='hdd'
                except Exception, detail:
                    print detail
                    device='cdrom'
            if (major == 8):
                device='hdd'
            if major == 9:
                device='tape'
            if major == 11:
                device='cdrom'
        return device
        
if __name__ == "__main__":
    a = QApplication(sys.argv)
    QObject.connect(a,SIGNAL("lastWindowClosed()"),a,SLOT("quit()"))
    w = main_kouvert()
    a.setMainWidget(w)
    w.show()
    a.exec_loop()
